@@ -55,13 +55,17 @@ my @opt_include_env = ();
 my $opt_testenv = 0;
 my $opt_list = 0;
 my $opt_mitkrb5 = 0;
+my $opt_default_ldb_backend = "mdb";
 my $opt_resetup_env = undef;
 my $opt_load_list = undef;
 my $opt_libnss_wrapper_so_path = "";
 my $opt_libresolv_wrapper_so_path = "";
 my $opt_libsocket_wrapper_so_path = "";
 my $opt_libuid_wrapper_so_path = "";
+my $opt_libpam_wrapper_so_path = "";
+my $opt_libpam_matrix_so_path = "";
 my $opt_libasan_so_path = "";
+my $opt_libcrypt_so_path = "";
 my $opt_use_dns_faking = 0;
 my @testlists = ();
 
@@ -124,10 +128,15 @@ sub run_testsuite($$$$$)
 	my ($envname, $name, $cmd, $i, $totalsuites) = @_;
 	my $pcap_file = $target->setup_pcap($name);
 
+	Subunit::report_time();
 	Subunit::start_testsuite($name);
 	Subunit::progress_push();
 	Subunit::report_time();
-	system($cmd);
+	# Enable pipefail so that we catch failing testsuites that are part of a
+	# pipeline (typically, piped through filter-subunit). This won't catch
+	# any testsuite failures that are turned into testsuite-xfails by
+	# filter-subunit.
+	system("bash", "-o", "pipefail", "-c", $cmd);
 	Subunit::report_time();
 	Subunit::progress_pop();
 
@@ -208,7 +217,7 @@ DNS:
 
 Target Specific:
  --socket-wrapper-pcap      save traffic to pcap directories
- --socket-wrapper-keep-pcap keep all pcap files, not just those for tests that 
+ --socket-wrapper-keep-pcap keep all pcap files, not just those for tests that
                             failed
  --socket-wrapper           enable socket wrapper
 
@@ -239,6 +248,7 @@ my $result = GetOptions (
 		'testenv' => \$opt_testenv,
 		'list' => \$opt_list,
 		'mitkrb5' => \$opt_mitkrb5,
+		'default-ldb-backend=s' => \$opt_default_ldb_backend,
 		'resetup-environment' => \$opt_resetup_env,
 		'testlist=s' => \@testlists,
 		'random-order' => \$opt_random_order,
@@ -247,7 +257,10 @@ my $result = GetOptions (
 		'resolv_wrapper_so_path=s' => \$opt_libresolv_wrapper_so_path,
 		'socket_wrapper_so_path=s' => \$opt_libsocket_wrapper_so_path,
 		'uid_wrapper_so_path=s' => \$opt_libuid_wrapper_so_path,
+		'pam_wrapper_so_path=s' => \$opt_libpam_wrapper_so_path,
+		'pam_matrix_so_path=s' => \$opt_libpam_matrix_so_path,
 		'asan_so_path=s' => \$opt_libasan_so_path,
+		'crypt_so_path=s' => \$opt_libcrypt_so_path,
 		'use-dns-faking' => \$opt_use_dns_faking
 	    );
 
@@ -271,10 +284,13 @@ unless (defined($ENV{VALGRIND})) {
 # make all our python scripts unbuffered
 $ENV{PYTHONUNBUFFERED} = 1;
 
+$ENV{SAMBA_DEPRECATED_SUPPRESS} = 1;
+
 # do not depend on the users setup
 # see also bootstrap/config.py
 $ENV{TZ} = "UTC";
 $ENV{LC_ALL} = $ENV{LANG} = "en_US.utf8";
+$ENV{LANGUAGE} = "en_US";
 
 my $bindir_abs = abs_path($bindir);
 
@@ -344,9 +360,17 @@ my $ld_preload = $ENV{LD_PRELOAD};
 
 if ($opt_libasan_so_path) {
 	if ($ld_preload) {
-		$ld_preload = "$ld_preload:$opt_libasan_so_path";
+		if ($opt_libcrypt_so_path) {
+			$ld_preload = "$opt_libasan_so_path:$opt_libcrypt_so_path:$ld_preload";
+		} else {
+			$ld_preload = "$opt_libasan_so_path:$ld_preload";
+		}
 	} else {
-		$ld_preload = "$opt_libasan_so_path";
+		if ($opt_libcrypt_so_path) {
+			$ld_preload = "$opt_libasan_so_path:$opt_libcrypt_so_path";
+		} else {
+			$ld_preload = "$opt_libasan_so_path";
+		}
 	}
 }
 
@@ -382,6 +406,14 @@ if ($opt_libuid_wrapper_so_path) {
 	}
 }
 
+if ($opt_libpam_wrapper_so_path) {
+	if ($ld_preload) {
+		$ld_preload = "$ld_preload:$opt_libpam_wrapper_so_path";
+	} else {
+		$ld_preload = "$opt_libpam_wrapper_so_path";
+	}
+}
+
 if (defined($ENV{USE_NAMESPACES})) {
 	print "Using linux containerization for selftest testenv(s)...\n";
 
@@ -401,12 +433,6 @@ $ENV{UID_WRAPPER} = 1;
 # We are already hitting the limit, so double it.
 $ENV{NSS_WRAPPER_MAX_HOSTENTS} = 200;
 
-# Disable RTLD_DEEPBIND hack for Samba bind dlz module
-#
-# This is needed in order to allow the ldb_*ldap module
-# to work with a preloaded socket wrapper.
-$ENV{LDB_MODULES_DISABLE_DEEPBIND} = 1;
-
 my $socket_wrapper_dir;
 if ($opt_socket_wrapper) {
 	$socket_wrapper_dir = SocketWrapper::setup_dir("$prefix_abs/w", $opt_socket_wrapper_pcap);
@@ -429,6 +455,10 @@ if ($opt_mitkrb5 == 1) {
 	$ENV{KRB5RCACHETYPE} = "none";
 }
 
+# Enable support for SHA1 in OpenSSL
+# This is required e.g. for pkinit sha1 tests
+$ENV{OPENSSL_ENABLE_SHA1_SIGNATURES} = 1;
+
 # After this many seconds, the server will self-terminate.  All tests
 # must terminate in this time, and testenv will only stay alive this
 # long
@@ -448,7 +478,9 @@ if (defined($ENV{SMBD_MAXTIME}) and $ENV{SMBD_MAXTIME} ne "") {
 
 $target = new Samba($bindir, $srcdir, $server_maxtime,
 		    $opt_socket_wrapper_pcap,
-		    $opt_socket_wrapper_keep_pcap);
+		    $opt_socket_wrapper_keep_pcap,
+		    $opt_libpam_matrix_so_path,
+		    $opt_default_ldb_backend);
 unless ($opt_list) {
 	if ($opt_target eq "samba") {
 		$testenv_default = "ad_dc";
@@ -462,14 +494,14 @@ sub read_test_regexes($)
 	my ($name) = @_;
 	my @ret = ();
 	open(LF, "<$name") or die("unable to read $name: $!");
-	while (<LF>) { 
-		chomp; 
+	while (<LF>) {
+		chomp;
 		next if (/^#/);
 		if (/^(.*?)([ \t]+)\#([\t ]*)(.*?)$/) {
 			push (@ret, [$1, $4]);
 		} else {
 			s/^(.*?)([ \t]+)\#([\t ]*)(.*?)$//;
-			push (@ret, [$_, undef]); 
+			push (@ret, [$_, undef]);
 		}
 	}
 	close(LF);
@@ -528,11 +560,12 @@ sub write_clientconf($$$)
 	my $cacert = "$cadir/Public/CA-samba.example.com-cert.pem";
 	my $cacrl_pem = "$cadir/Public/CA-samba.example.com-crl.pem";
 	my $ca_users_dir = "$cadir/Users";
+	my $client_loglevel = $ENV{CLIENT_LOG_LEVEL} || 1;
 
 	# each user has a USER-${USER_PRINCIPAL_NAME}-cert.pem and
 	# USER-${USER_PRINCIPAL_NAME}-private-key.pem symlink
 	# We make a copy here and make the certificated easily
-	# accessable in the client environment.
+	# accessible in the client environment.
 	my $mask = umask;
 	umask 0077;
 	opendir USERS, "${ca_users_dir}" or die "Could not open dir '${ca_users_dir}': $!";
@@ -581,7 +614,7 @@ sub write_clientconf($$$)
 	system:anonymous = true
 	client lanman auth = Yes
 	client min protocol = CORE
-	log level = 1
+	log level = $client_loglevel
 	torture:basedir = $clientdir
 #We don't want to run 'speed' tests for very long
         torture:timelimit = 1
@@ -978,7 +1011,7 @@ $envvarstr
 			next;
 		}
 
-		# Generate a file with the individual tests to run, if the 
+		# Generate a file with the individual tests to run, if the
 		# test runner for this test suite supports it.
 		if ($individual_tests and $individual_tests->{$name}) {
 			if ($$_[3]) {
