@@ -57,13 +57,14 @@ struct smb_filename *synthetic_smb_fname(TALLOC_CTX *mem_ctx,
 					 NTTIME twrp,
 					 uint32_t flags)
 {
-	struct smb_filename smb_fname_loc = { 0, };
-
 	/* Setup the base_name/stream_name. */
-	smb_fname_loc.base_name = discard_const_p(char, base_name);
-	smb_fname_loc.stream_name = discard_const_p(char, stream_name);
-	smb_fname_loc.flags = flags;
-	smb_fname_loc.twrp = twrp;
+
+	struct smb_filename smb_fname_loc = {
+		.base_name = discard_const_p(char, base_name),
+		.stream_name = discard_const_p(char, stream_name),
+		.flags = flags,
+		.twrp = twrp,
+	};
 
 	/* Copy the psbuf if one was given. */
 	if (psbuf)
@@ -80,12 +81,12 @@ struct smb_filename *synthetic_smb_fname(TALLOC_CTX *mem_ctx,
 struct smb_filename *cp_smb_filename_nostream(TALLOC_CTX *mem_ctx,
 					const struct smb_filename *smb_fname_in)
 {
-	struct smb_filename *smb_fname = cp_smb_filename(mem_ctx,
-							smb_fname_in);
-	if (smb_fname == NULL) {
-		return NULL;
-	}
-	TALLOC_FREE(smb_fname->stream_name);
+	struct smb_filename smb_fname_loc = *smb_fname_in;
+	struct smb_filename *smb_fname = NULL;
+
+	smb_fname_loc.stream_name = NULL;
+
+	smb_fname = cp_smb_filename(mem_ctx, &smb_fname_loc);
 	return smb_fname;
 }
 
@@ -165,10 +166,8 @@ const char *smb_fname_str_dbg(const struct smb_filename *smb_fname)
 		return "";
 	}
 
-	fname = talloc_asprintf(talloc_tos(),
-				"%s {%s}",
-				fname,
-				tstr);
+	fname = talloc_asprintf_append_buffer(
+		fname, " {%s}", tstr);
 	if (fname == NULL) {
 		return "";
 	}
@@ -187,21 +186,6 @@ const char *fsp_str_dbg(const struct files_struct *fsp)
 		return "";
 	}
 
-	if (fsp->dirfsp == NULL || fsp->dirfsp == fsp->conn->cwd_fsp) {
-		return name;
-	}
-
-	if (ISDOT(fsp->dirfsp->fsp_name->base_name)) {
-		return name;
-	}
-
-	name = talloc_asprintf(talloc_tos(),
-			       "%s/%s",
-			       fsp->dirfsp->fsp_name->base_name,
-			       fsp->fsp_name->base_name);
-	if (name == NULL) {
-		return "";
-	}
 	return name;
 }
 
@@ -224,8 +208,7 @@ const char *fsp_fnum_dbg(const struct files_struct *fsp)
 		return "fnum [invalid value]";
 	}
 
-	str = talloc_asprintf(talloc_tos(), "fnum %llu",
-			      (unsigned long long)fsp->fnum);
+	str = talloc_asprintf(talloc_tos(), "fnum %"PRIu64, fsp->fnum);
 	if (str == NULL) {
 		DEBUG(1, ("%s: talloc_asprintf failed\n", __FUNCTION__));
 		return "fnum [talloc failed!]";
@@ -286,69 +269,6 @@ struct smb_filename *cp_smb_filename(TALLOC_CTX *mem_ctx,
 	return out;
 }
 
-/**
- * Return allocated parent directory and basename of path
- *
- * Note: if requesting name, it is returned as talloc child of the
- * parent. Freeing the parent is thus sufficient to free both.
- */
-bool parent_smb_fname(TALLOC_CTX *mem_ctx,
-		      const struct smb_filename *path,
-		      struct smb_filename **_parent,
-		      struct smb_filename  **_name)
-{
-	TALLOC_CTX *frame = talloc_stackframe();
-	struct smb_filename *parent = NULL;
-	struct smb_filename *name = NULL;
-	char *p = NULL;
-
-	parent = cp_smb_filename(frame, path);
-	if (parent == NULL) {
-		TALLOC_FREE(frame);
-		return false;
-	}
-	TALLOC_FREE(parent->stream_name);
-	SET_STAT_INVALID(parent->st);
-
-	p = strrchr_m(parent->base_name, '/'); /* Find final '/', if any */
-	if (p == NULL) {
-		TALLOC_FREE(parent->base_name);
-		parent->base_name = talloc_strdup(parent, ".");
-		if (parent->base_name == NULL) {
-			TALLOC_FREE(frame);
-			return false;
-		}
-		p = path->base_name;
-	} else {
-		*p = '\0';
-		p++;
-	}
-
-	if (_name == NULL) {
-		*_parent = talloc_move(mem_ctx, &parent);
-		TALLOC_FREE(frame);
-		return true;
-	}
-
-	name = cp_smb_filename(frame, path);
-	if (name == NULL) {
-		TALLOC_FREE(frame);
-		return false;
-	}
-	TALLOC_FREE(name->base_name);
-
-	name->base_name = talloc_strdup(name, p);
-	if (name == NULL) {
-		TALLOC_FREE(frame);
-		return false;
-	}
-
-	*_parent = talloc_move(mem_ctx, &parent);
-	*_name = talloc_move(*_parent, &name);
-	TALLOC_FREE(frame);
-	return true;
-}
-
 static void assert_valid_stream_smb_fname(const struct smb_filename *smb_fname)
 {
 	/* stream_name must always be NULL if there is no stream. */
@@ -370,11 +290,7 @@ bool is_ntfs_stream_smb_fname(const struct smb_filename *smb_fname)
 {
 	assert_valid_stream_smb_fname(smb_fname);
 
-	if (smb_fname->stream_name == NULL) {
-		return false;
-	}
-
-	return true;
+	return (smb_fname->stream_name != NULL);
 }
 
 /****************************************************************************
@@ -421,7 +337,7 @@ bool is_ntfs_default_stream_smb_fname(const struct smb_filename *smb_fname)
  Filter out Windows invalid EA names (list probed from Windows 2012).
 ****************************************************************************/
 
-static char bad_ea_name_chars[] = "\"*+,/:;<=>?[\\]|";
+static const char bad_ea_name_chars[] = "\"*+,/:;<=>?[\\]|";
 
 bool is_invalid_windows_ea_name(const char *name)
 {
