@@ -70,7 +70,6 @@ static struct tevent_req *cli_message_start_send(TALLOC_CTX *mem_ctx,
 	p += ulen;
 	*p++ = 4;
 	memcpy(p, htmp, hlen);
-	p += hlen;
 	TALLOC_FREE(htmp);
 	TALLOC_FREE(utmp);
 
@@ -101,9 +100,7 @@ static void cli_message_start_done(struct tevent_req *subreq)
 	status = cli_smb_recv(subreq, state, NULL, 0, &wct, &vwv,
 			      NULL, NULL);
 	TALLOC_FREE(subreq);
-	if (!NT_STATUS_IS_OK(status)) {
-		TALLOC_FREE(subreq);
-		tevent_req_nterror(req, status);
+	if (tevent_req_nterror(req, status)) {
 		return;
 	}
 	if (wct >= 1) {
@@ -192,8 +189,7 @@ static void cli_message_text_done(struct tevent_req *subreq)
 
 	status = cli_smb_recv(subreq, NULL, NULL, 0, NULL, NULL, NULL, NULL);
 	TALLOC_FREE(subreq);
-	if (!NT_STATUS_IS_OK(status)) {
-		tevent_req_nterror(req, status);
+	if (tevent_req_nterror(req, status)) {
 		return;
 	}
 	tevent_req_done(req);
@@ -243,8 +239,7 @@ static void cli_message_end_done(struct tevent_req *subreq)
 
 	status = cli_smb_recv(subreq, NULL, NULL, 0, NULL, NULL, NULL, NULL);
 	TALLOC_FREE(subreq);
-	if (!NT_STATUS_IS_OK(status)) {
-		tevent_req_nterror(req, status);
+	if (tevent_req_nterror(req, status)) {
 		return;
 	}
 	tevent_req_done(req);
@@ -304,8 +299,7 @@ static void cli_message_started(struct tevent_req *subreq)
 
 	status = cli_message_start_recv(subreq, &state->grp);
 	TALLOC_FREE(subreq);
-	if (!NT_STATUS_IS_OK(status)) {
-		tevent_req_nterror(req, status);
+	if (tevent_req_nterror(req, status)) {
 		return;
 	}
 
@@ -331,8 +325,7 @@ static void cli_message_sent(struct tevent_req *subreq)
 
 	status = cli_message_text_recv(subreq);
 	TALLOC_FREE(subreq);
-	if (!NT_STATUS_IS_OK(status)) {
-		tevent_req_nterror(req, status);
+	if (tevent_req_nterror(req, status)) {
 		return;
 	}
 
@@ -368,8 +361,7 @@ static void cli_message_done(struct tevent_req *subreq)
 
 	status = cli_message_end_recv(subreq);
 	TALLOC_FREE(subreq);
-	if (!NT_STATUS_IS_OK(status)) {
-		tevent_req_nterror(req, status);
+	if (tevent_req_nterror(req, status)) {
 		return;
 	}
 	tevent_req_done(req);
