@@ -18,8 +18,10 @@
 */
 
 #include "includes.h"
+#include "lib/util/util_file.h"
 #include "printing.h"
 #include "smbd/proto.h"
+#include "source3/lib/substitute.h"
 
 extern userdom_struct current_user_info;
 
