@@ -26,7 +26,7 @@
 #include "includes.h"
 #include "system/filesys.h"
 #include "lib/util/server_id.h"
-#include "popt_common.h"
+#include "lib/cmdline/cmdline.h"
 #include "librpc/gen_ndr/spoolss.h"
 #include "nt_printing.h"
 #include "printing/notify.h"
@@ -35,9 +35,11 @@
 #include "util_tdb.h"
 #include "../lib/util/pidfile.h"
 #include "serverid.h"
+#include "lib/util/server_id_db.h"
 #include "cmdline_contexts.h"
 #include "lib/util/string_wrappers.h"
 #include "lib/global_contexts.h"
+#include "lib/param/param.h"
 
 #ifdef HAVE_LIBUNWIND_H
 #include <libunwind.h>
@@ -115,8 +117,8 @@ static void wait_replies(struct tevent_context *ev_ctx,
 /* Message handler callback that displays the PID and a string on stdout */
 
 static void print_pid_string_cb(struct messaging_context *msg,
-				void *private_data, 
-				uint32_t msg_type, 
+				void *private_data,
+				uint32_t msg_type,
 				struct server_id pid,
 				DATA_BLOB *data)
 {
@@ -457,8 +459,8 @@ static bool do_election(struct tevent_context *ev_ctx,
 /* Ping a samba daemon process */
 
 static void pong_cb(struct messaging_context *msg,
-		    void *private_data, 
-		    uint32_t msg_type, 
+		    void *private_data,
+		    uint32_t msg_type,
 		    struct server_id pid,
 		    DATA_BLOB *data)
 {
@@ -530,8 +532,8 @@ static bool do_profile(struct tevent_context *ev_ctx,
 /* Return the profiling level */
 
 static void profilelevel_cb(struct messaging_context *msg_ctx,
-			    void *private_data, 
-			    uint32_t msg_type, 
+			    void *private_data,
+			    uint32_t msg_type,
 			    struct server_id pid,
 			    DATA_BLOB *data)
 {
@@ -541,7 +543,7 @@ static void profilelevel_cb(struct messaging_context *msg_ctx,
 	num_replies++;
 
 	if (data->length != sizeof(int)) {
-		fprintf(stderr, "invalid message length %ld returned\n", 
+		fprintf(stderr, "invalid message length %ld returned\n",
 			(unsigned long)data->length);
 		return;
 	}
@@ -570,8 +572,8 @@ static void profilelevel_cb(struct messaging_context *msg_ctx,
 }
 
 static void profilelevel_rqst(struct messaging_context *msg_ctx,
-			      void *private_data, 
-			      uint32_t msg_type, 
+			      void *private_data,
+			      uint32_t msg_type,
 			      struct server_id pid,
 			      DATA_BLOB *data)
 {
@@ -727,7 +729,7 @@ static bool do_printnotify(struct tevent_context *ev_ctx,
 
 		notify_job_status_byname(
 			ev_ctx, msg_ctx,
-			argv[2], jobid, JOB_STATUS_QUEUED, 
+			argv[2], jobid, JOB_STATUS_QUEUED,
 			SPOOLSS_NOTIFY_MSG_UNIX_JOBID);
 
 		goto send;
@@ -903,6 +905,39 @@ static bool do_poolusage(struct tevent_context *ev_ctx,
 	return true;
 }
 
+static bool do_rpc_dump_status(
+	struct tevent_context *ev_ctx,
+	struct messaging_context *msg_ctx,
+	const struct server_id dst,
+	const int argc,
+	const char **argv)
+{
+	pid_t pid = procid_to_pid(&dst);
+	int stdout_fd = 1;
+
+	if (argc != 1) {
+		fprintf(stderr,
+			"Usage: smbcontrol <dest> rpc-dump-status\n");
+		return False;
+	}
+
+	if (pid == 0) {
+		fprintf(stderr, "Can only send to a specific PID\n");
+		return false;
+	}
+
+	messaging_send_iov(
+		msg_ctx,
+		dst,
+		MSG_RPC_DUMP_STATUS,
+		NULL,
+		0,
+		&stdout_fd,
+		1);
+
+	return true;
+}
+
 /* Fetch and print the ringbuf log */
 
 static void print_ringbuf_log_cb(struct messaging_context *msg,
@@ -1148,7 +1183,7 @@ static bool do_winbind_offline(struct tevent_context *ev_ctx,
 
 	/* There's a potential race condition that if a child
 	   winbindd detects a domain is online at the same time
-	   we're trying to tell it to go offline that it might 
+	   we're trying to tell it to go offline that it might
 	   delete the record we add between us adding it and
 	   sending the message. Minimize this by retrying up to
 	   5 times. */
@@ -1308,6 +1343,18 @@ static bool do_winbind_validate_cache(struct tevent_context *ev_ctx,
 	return num_replies;
 }
 
+static bool do_reload_certs(struct tevent_context *ev_ctx,
+					struct messaging_context *msg_ctx,
+					const struct server_id pid,
+					const int argc, const char **argv)
+{
+	if (argc != 1) {
+		fprintf(stderr, "Usage: smbcontrol ldap_server reload-certs \n");
+		return false;
+	}
+
+	return send_message(msg_ctx, pid, MSG_RELOAD_TLS_CERTIFICATES, NULL, 0);
+}
 static bool do_reload_config(struct tevent_context *ev_ctx,
 			     struct messaging_context *msg_ctx,
 			     const struct server_id pid,
@@ -1479,6 +1526,11 @@ static const struct {
 		.fn   = do_poolusage,
 		.help = "Display talloc memory usage",
 	},
+	{
+		.name = "rpc-dump-status",
+		.fn   = do_rpc_dump_status,
+		.help = "Display rpc status",
+	},
 	{
 		.name = "ringbuf-log",
 		.fn   = do_ringbuflog,
@@ -1504,6 +1556,11 @@ static const struct {
 		.fn   = do_drvupgrade,
 		.help = "Notify a printer driver has changed",
 	},
+	{
+		.name = "reload-certs",
+		.fn   = do_reload_certs,
+		.help = "Reload TLS certificates"
+	},
 	{
 		.name = "reload-config",
 		.fn   = do_reload_config,
@@ -1576,8 +1633,8 @@ static void usage(poptContext pc)
 	poptPrintHelp(pc, stderr, 0);
 
 	fprintf(stderr, "\n");
-	fprintf(stderr, "<destination> is one of \"nmbd\", \"smbd\", \"winbindd\" or a "
-		"process ID\n");
+	fprintf(stderr, "<destination> is one of \"nmbd\", \"smbd\", \"winbindd\", "
+		"\"ldap_server\" or a process ID\n");
 
 	fprintf(stderr, "\n");
 	fprintf(stderr, "<message-type> is one of:\n");
@@ -1604,6 +1661,8 @@ static struct server_id parse_dest(struct messaging_context *msg,
 		.pid = (uint64_t)-1,
 	};
 	pid_t pid;
+	struct server_id_db *names_db = NULL;
+	bool ok;
 
 	/* Zero is a special return value for broadcast to all processes */
 
@@ -1636,6 +1695,16 @@ static struct server_id parse_dest(struct messaging_context *msg,
 		return pid_to_procid(pid);
 	}
 
+	names_db = messaging_names_db(msg);
+	if (names_db == NULL) {
+		goto fail;
+	}
+	ok = server_id_db_lookup_one(names_db, dest, &result);
+	if (ok) {
+		return result;
+	}
+
+fail:
 	fprintf(stderr,"Can't find pid for destination '%s'\n", dest);
 
 	return result;
@@ -1703,28 +1772,47 @@ int main(int argc, const char **argv)
 	struct tevent_context *evt_ctx;
 	struct messaging_context *msg_ctx;
 
-	static struct poptOption long_options[] = {
+	struct poptOption long_options[] = {
 		/* POPT_AUTOHELP */
 		{ NULL, '\0', POPT_ARG_INCLUDE_TABLE, help_options,
 		                        0, "Help options:", NULL },
-		{ "timeout", 't', POPT_ARG_INT, &timeout, 't', 
+		{ "timeout", 't', POPT_ARG_INT, &timeout, 't',
 		  "Set timeout value in seconds", "TIMEOUT" },
 
 		POPT_COMMON_SAMBA
+		POPT_COMMON_VERSION
 		POPT_TABLEEND
 	};
 	TALLOC_CTX *frame = talloc_stackframe();
+	struct loadparm_context *lp_ctx = NULL;
 	int ret = 0;
+	bool ok;
 
 	smb_init_locale();
 
-	setup_logging(argv[0], DEBUG_STDOUT);
-	lp_set_cmdline("log level", "0");
+	ok = samba_cmdline_init(frame,
+				SAMBA_CMDLINE_CONFIG_CLIENT,
+				false /* require_smbconf */);
+	if (!ok) {
+		DBG_ERR("Failed to init cmdline parser!\n");
+		TALLOC_FREE(frame);
+		exit(1);
+	}
+	lp_ctx = samba_cmdline_get_lp_ctx();
+	lpcfg_set_cmdline(lp_ctx, "log level", "0");
 
 	/* Parse command line arguments using popt */
 
-	pc = poptGetContext(
-		"smbcontrol", argc, (const char **)argv, long_options, 0);
+	pc = samba_popt_get_context(getprogname(),
+				    argc,
+				    argv,
+				    long_options,
+				    0);
+	if (pc == NULL) {
+		DBG_ERR("Failed to setup popt context!\n");
+		TALLOC_FREE(frame);
+		exit(1);
+	}
 
 	poptSetOtherOptionHelp(pc, "[OPTION...] <destination> <message-type> "
 			       "<parameters>");
@@ -1768,15 +1856,14 @@ int main(int argc, const char **argv)
 
 	evt_ctx = global_event_context();
 
-	lp_load_global(get_dyn_CONFIGFILE());
-
 	/* Need to invert sense of return code -- samba
          * routines mostly return True==1 for success, but
-         * shell needs 0. */ 
+         * shell needs 0. */
 
 	ret = !do_command(evt_ctx, msg_ctx, argc, argv);
 
 	cmdline_messaging_context_free();
+	gfree_all();
 	poptFreeContext(pc);
 	TALLOC_FREE(frame);
 	return ret;
