@@ -1,21 +1,21 @@
-/* 
+/*
    Unix SMB/CIFS implementation.
 
    Kerberos utility functions for GENSEC
-   
+
    Copyright (C) Andrew Bartlett <abartlet@samba.org> 2004-2005
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    (at your option) any later version.
-   
+
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 
-   
+
    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
@@ -24,7 +24,6 @@
 #include "system/kerberos.h"
 #include "auth/kerberos/kerberos.h"
 #include "auth/credentials/credentials.h"
-#include "auth/credentials/credentials_proto.h"
 #include "auth/credentials/credentials_krb5.h"
 #include "auth/kerberos/kerberos_credentials.h"
 #include "auth/kerberos/kerberos_util.h"
@@ -57,6 +56,24 @@ static krb5_error_code parse_principal(TALLOC_CTX *parent_ctx,
 		 return 0;
 	}
 
+	/*
+	 * Start with talloc(), talloc_reference() and only then call
+	 * krb5_parse_name(). If any of them fails, the cleanup code is simpler.
+	 */
+	mem_ctx = talloc(parent_ctx, struct principal_container);
+	if (!mem_ctx) {
+		(*error_string) = error_message(ENOMEM);
+		return ENOMEM;
+	}
+
+	mem_ctx->smb_krb5_context = talloc_reference(mem_ctx,
+						     smb_krb5_context);
+	if (mem_ctx->smb_krb5_context == NULL) {
+		(*error_string) = error_message(ENOMEM);
+		talloc_free(mem_ctx);
+		return ENOMEM;
+	}
+
 	ret = krb5_parse_name(smb_krb5_context->krb5_context,
 			      princ_string, princ);
 
@@ -64,19 +81,12 @@ static krb5_error_code parse_principal(TALLOC_CTX *parent_ctx,
 		(*error_string) = smb_get_krb5_error_message(
 						smb_krb5_context->krb5_context,
 						ret, parent_ctx);
+		talloc_free(mem_ctx);
 		return ret;
 	}
 
-	mem_ctx = talloc(parent_ctx, struct principal_container);
-	if (!mem_ctx) {
-		(*error_string) = error_message(ENOMEM);
-		return ENOMEM;
-	}
-
-	/* This song-and-dance effectivly puts the principal
-	 * into talloc, so we can't loose it. */
-	mem_ctx->smb_krb5_context = talloc_reference(mem_ctx,
-						     smb_krb5_context);
+	/* This song-and-dance effectively puts the principal
+	 * into talloc, so we can't lose it. */
 	mem_ctx->principal = *princ;
 	talloc_set_destructor(mem_ctx, free_principal);
 	return 0;
@@ -222,12 +232,13 @@ done:
 
 /**
  * Return a freshly allocated ccache (destroyed by destructor on child
- * of parent_ctx), for a given set of client credentials 
+ * of parent_ctx), for a given set of client credentials
  */
 
  krb5_error_code kinit_to_ccache(TALLOC_CTX *parent_ctx,
 				 struct cli_credentials *credentials,
 				 struct smb_krb5_context *smb_krb5_context,
+				 struct loadparm_context *lp_ctx,
 				 struct tevent_context *event_ctx,
 				 krb5_ccache ccache,
 				 enum credentials_obtained *obtained,
@@ -235,9 +246,7 @@ done:
 {
 	krb5_error_code ret;
 	const char *password;
-#ifdef SAMBA4_USES_HEIMDAL
 	const char *self_service;
-#endif
 	const char *target_service;
 	time_t kdc_time = 0;
 	krb5_principal princ;
@@ -245,6 +254,7 @@ done:
 	int tries;
 	TALLOC_CTX *mem_ctx = talloc_new(parent_ctx);
 	krb5_get_init_creds_opt *krb_options;
+	struct cli_credentials *fast_creds;
 
 	if (!mem_ctx) {
 		(*error_string) = strerror(ENOMEM);
@@ -269,9 +279,7 @@ done:
 		return ret;
 	}
 
-#ifdef SAMBA4_USES_HEIMDAL
 	self_service = cli_credentials_get_self_service(credentials);
-#endif
 	target_service = cli_credentials_get_target_service(credentials);
 
 	password = cli_credentials_get_password(credentials);
@@ -313,10 +321,56 @@ done:
 	 */
 	krb5_get_init_creds_opt_set_win2k(smb_krb5_context->krb5_context,
 					  krb_options, true);
+	krb5_get_init_creds_opt_set_canonicalize(smb_krb5_context->krb5_context,
+						 krb_options, true);
 #else /* MIT */
 	krb5_get_init_creds_opt_set_canonicalize(krb_options, true);
 #endif
 
+	fast_creds = cli_credentials_get_krb5_fast_armor_credentials(credentials);
+
+	if (fast_creds != NULL) {
+#ifdef HAVE_KRB5_GET_INIT_CREDS_OPT_SET_FAST_CCACHE
+		struct ccache_container *fast_ccc = NULL;
+		const char *fast_error_string = NULL;
+		ret = cli_credentials_get_ccache(fast_creds, event_ctx, lp_ctx, &fast_ccc, &fast_error_string);
+		if (ret != 0) {
+			(*error_string) = talloc_asprintf(credentials,
+							  "Obtaining the Kerberos FAST armor credentials failed: %s\n",
+							  fast_error_string);
+			return ret;
+		}
+		krb5_get_init_creds_opt_set_fast_ccache(smb_krb5_context->krb5_context,
+							krb_options,
+							fast_ccc->ccache);
+#else
+		*error_string = talloc_strdup(credentials,
+					      "Using Kerberos FAST "
+					      "armor credentials not possible "
+					      "with this Kerberos library.  "
+					      "Modern MIT or Samba's embedded "
+					      "Heimdal required");
+		return EINVAL;
+#endif
+	}
+
+#ifdef HAVE_KRB5_GET_INIT_CREDS_OPT_SET_FAST_FLAGS
+	{
+		bool require_fast;
+		/*
+		 * This ensures that if FAST was required, but no armor
+		 * credentials cache was specified, we proceed with (eg)
+		 * anonymous PKINIT
+		 */
+		require_fast = cli_credentials_get_krb5_require_fast_armor(credentials);
+		if (require_fast) {
+			krb5_get_init_creds_opt_set_fast_flags(smb_krb5_context->krb5_context,
+							       krb_options,
+							       KRB5_FAST_REQUIRED);
+		}
+	}
+#endif
+
 	tries = 2;
 	while (tries--) {
 #ifdef SAMBA4_USES_HEIMDAL
@@ -325,12 +379,12 @@ done:
 		ret = smb_krb5_context_set_event_ctx(smb_krb5_context, event_ctx, &previous_ev);
 		if (ret) {
 			talloc_free(mem_ctx);
+			krb5_get_init_creds_opt_free(smb_krb5_context->krb5_context, krb_options);
 			return ret;
 		}
 #endif
 		if (password) {
 			if (impersonate_principal) {
-#ifdef SAMBA4_USES_HEIMDAL
 				ret = smb_krb5_kinit_s4u2_ccache(smb_krb5_context->krb5_context,
 								 ccache,
 								 princ,
@@ -341,12 +395,6 @@ done:
 								 krb_options,
 								 NULL,
 								 &kdc_time);
-#else
-				talloc_free(mem_ctx);
-				(*error_string) = "INTERNAL error: s4u2 ops "
-					"are not supported with MIT build yet";
-				return EINVAL;
-#endif
 			} else {
 				ret = smb_krb5_kinit_password_ccache(smb_krb5_context->krb5_context,
 								     ccache,
@@ -360,10 +408,14 @@ done:
 		} else if (impersonate_principal) {
 			talloc_free(mem_ctx);
 			(*error_string) = "INTERNAL error: Cannot impersonate principal with just a keyblock.  A password must be specified in the credentials";
+			krb5_get_init_creds_opt_free(smb_krb5_context->krb5_context, krb_options);
+#ifdef SAMBA4_USES_HEIMDAL
+			smb_krb5_context_remove_event_ctx(smb_krb5_context, previous_ev, event_ctx);
+#endif
 			return EINVAL;
 		} else {
 			/* No password available, try to use a keyblock instead */
-			
+
 			krb5_keyblock keyblock;
 			const struct samr_Password *mach_pwd;
 			mach_pwd = cli_credentials_get_nt_hash(credentials, mem_ctx);
@@ -378,9 +430,9 @@ done:
 			}
 			ret = smb_krb5_keyblock_init_contents(smb_krb5_context->krb5_context,
 						 ENCTYPE_ARCFOUR_HMAC,
-						 mach_pwd->hash, sizeof(mach_pwd->hash), 
+						 mach_pwd->hash, sizeof(mach_pwd->hash),
 						 &keyblock);
-			
+
 			if (ret == 0) {
 				ret = smb_krb5_kinit_keyblock_ccache(smb_krb5_context->krb5_context,
 								     ccache,
@@ -426,11 +478,12 @@ done:
 		DEBUG(4,("Advancing clock by %d seconds to cope with clock skew\n", time_offset));
 		krb5_set_real_time(smb_krb5_context->krb5_context, t + time_offset + 1, 0);
 	}
-	
+
 	if (ret == KRB5KDC_ERR_PREAUTH_FAILED && cli_credentials_wrong_password(credentials)) {
 		ret = kinit_to_ccache(parent_ctx,
 				      credentials,
 				      smb_krb5_context,
+				      lp_ctx,
 				      event_ctx,
 				      ccache, obtained,
 				      error_string);
@@ -467,6 +520,21 @@ krb5_error_code smb_krb5_get_keytab_container(TALLOC_CTX *mem_ctx,
 	krb5_keytab keytab;
 	krb5_error_code ret;
 
+	/*
+	 * Start with talloc(), talloc_reference() and only then call
+	 * krb5_kt_resolve(). If any of them fails, the cleanup code is simpler.
+	 */
+	*ktc = talloc(mem_ctx, struct keytab_container);
+	if (!*ktc) {
+		return ENOMEM;
+	}
+
+	(*ktc)->smb_krb5_context = talloc_reference(*ktc, smb_krb5_context);
+	if ((*ktc)->smb_krb5_context == NULL) {
+		TALLOC_FREE(*ktc);
+		return ENOMEM;
+	}
+
 	if (opt_keytab) {
 		keytab = opt_keytab;
 	} else {
@@ -477,16 +545,11 @@ krb5_error_code smb_krb5_get_keytab_container(TALLOC_CTX *mem_ctx,
 				 smb_get_krb5_error_message(
 					smb_krb5_context->krb5_context,
 					ret, mem_ctx)));
+			TALLOC_FREE(*ktc);
 			return ret;
 		}
 	}
 
-	*ktc = talloc(mem_ctx, struct keytab_container);
-	if (!*ktc) {
-		return ENOMEM;
-	}
-
-	(*ktc)->smb_krb5_context = talloc_reference(*ktc, smb_krb5_context);
 	(*ktc)->keytab = keytab;
 	(*ktc)->password_based = false;
 	talloc_set_destructor(*ktc, free_keytab_container);
@@ -539,7 +602,7 @@ krb5_error_code smb_krb5_remove_obsolete_keytab_entries(TALLOC_CTX *mem_ctx,
 	default:
 		*error_string = talloc_asprintf(mem_ctx,
 						"failed to open keytab for read of old entries: %s\n",
-						smb_get_krb5_error_message(context, code, mem_ctx));
+						smb_get_krb5_error_message(context, code, tmp_ctx));
 		goto done;
 	}
 
@@ -633,7 +696,7 @@ krb5_error_code smb_krb5_remove_obsolete_keytab_entries(TALLOC_CTX *mem_ctx,
 		krb5_kt_free_entry(context, &entry);
 		/* Make sure we do not double free */
 		ZERO_STRUCT(entry);
-	} while (code != 0);
+	} while (code == 0);
 
 	krb5_kt_end_seq_get(context, keytab, &cursor);
 
@@ -642,14 +705,117 @@ krb5_error_code smb_krb5_remove_obsolete_keytab_entries(TALLOC_CTX *mem_ctx,
 		break;
 	case ENOENT:
 	case KRB5_KT_END:
-		code = 0;
 		break;
 	default:
 		*error_string = talloc_asprintf(mem_ctx,
 						"failed in deleting old entries for principal: %s\n",
 						smb_get_krb5_error_message(context,
 									   code,
-									   mem_ctx));
+									   tmp_ctx));
+		goto done;
+	}
+
+	code = 0;
+done:
+	talloc_free(tmp_ctx);
+	return code;
+}
+
+/*
+ * Walk the keytab, looking for entries of this principal name,
+ * with KVNO and key equal
+ *
+ * These entries do not need to be replaced, so we want to tell the caller not to add them again
+ *
+ * Inspired by the code in Samba3 for 'use kerberos keytab'.
+ */
+krb5_error_code smb_krb5_is_exact_entry_in_keytab(TALLOC_CTX *mem_ctx,
+						  krb5_context context,
+						  krb5_keytab keytab,
+						  krb5_keytab_entry *to_match,
+						  bool *found,
+						  const char **error_string)
+{
+	TALLOC_CTX *tmp_ctx;
+	krb5_error_code code;
+	krb5_kt_cursor cursor;
+
+	tmp_ctx = talloc_new(mem_ctx);
+	if (tmp_ctx == NULL) {
+		*error_string = "Cannot allocate tmp_ctx";
+		return ENOMEM;
+	}
+
+	*found = false;
+
+	code = krb5_kt_start_seq_get(context, keytab, &cursor);
+	switch (code) {
+	case 0:
+		break;
+#ifdef HEIM_ERR_OPNOTSUPP
+	case HEIM_ERR_OPNOTSUPP:
+#endif
+	case ENOENT:
+	case KRB5_KT_END:
+		/* no point enumerating if there isn't anything here */
+		code = 0;
+		goto done;
+	default:
+		*error_string = talloc_asprintf(mem_ctx,
+						"failed to open keytab for read of existing entries: %s\n",
+						smb_get_krb5_error_message(context, code, tmp_ctx));
+		goto done;
+	}
+
+	do {
+		krb5_keytab_entry entry;
+		bool matched = false;
+		krb5_boolean ok;
+
+		code = krb5_kt_next_entry(context, keytab, &entry, &cursor);
+		if (code) {
+			break;
+		}
+
+		ok = smb_krb5_kt_compare(context,
+					 &entry,
+					 to_match->principal,
+					 to_match->vno,
+					 KRB5_KEY_TYPE(KRB5_KT_KEY(to_match)));
+		if (ok) {
+			/* This is not a security check, constant time is not required */
+			if ((KRB5_KEY_LENGTH(KRB5_KT_KEY(&entry)) == KRB5_KEY_LENGTH(KRB5_KT_KEY(to_match)))
+			    && memcmp(KRB5_KEY_DATA(KRB5_KT_KEY(&entry)), KRB5_KEY_DATA(KRB5_KT_KEY(to_match)),
+				      KRB5_KEY_LENGTH(KRB5_KT_KEY(&entry))) == 0) {
+				matched = true;
+			}
+		}
+
+		/* Free the entry, we don't need it any more */
+		krb5_kt_free_entry(context, &entry);
+		/* Make sure we do not double free */
+		ZERO_STRUCT(entry);
+		if (matched) {
+			*found = true;
+			break;
+		}
+	} while (code == 0);
+
+	krb5_kt_end_seq_get(context, keytab, &cursor);
+
+	switch (code) {
+	case 0:
+		break;
+	case ENOENT:
+	case KRB5_KT_END:
+		break;
+	default:
+		*error_string = talloc_asprintf(mem_ctx,
+						"failed in checking old entries for principal: %s\n",
+						smb_get_krb5_error_message(context,
+									   code,
+									   tmp_ctx));
+		goto done;
 	}
 
 	code = 0;
